import { AppBar } from "@/src/components/AppBar";
import { BottomSheet } from "@/src/components/BottomSheet";
import { Button } from "@/src/components/Button";
import { InputTextController } from "@/src/components/controllers/InputTextController";
import { InputTextMaskController } from "@/src/components/controllers/InputTextMaskController";
import { Message } from "@/src/components/Message";
import { Text } from "@/src/components/Text";
import { CreatePaymentWithCreditCardDto } from "@/src/models/payment.model";
import theme from "@/src/theme";
import { validateCpf } from "@/src/utils/validate-cpf";
import efiPay from "payment-token-efi";
import { useEffect, useRef, useState } from "react";
import { useForm } from "react-hook-form";
import WebView from "react-native-webview";

import { useAuth } from "@/src/hooks/authentication.hook";
import { useGlobalHook } from "@/src/hooks/global-hook";
import { ticketCreditCard } from "@/src/services/world-cup/ticket-payment.service";
import { router } from "expo-router";
import {
  ActivityIndicator,
  Alert,
  Platform,
  SafeAreaView,
  StatusBar,
  View,
} from "react-native";
import { KeyboardAwareScrollView } from "react-native-keyboard-aware-scroll-view";

export default function PaymentWithCreditCard() {
  const { profile, handleGetProfile } = useAuth();
  const { payloadToPaymentTickets } = useGlobalHook();
  const { ticket, daysSelect, metaData, totalValue } = payloadToPaymentTickets;
  const [getLoading, setGetLoading] = useState(false);
  const [success, setSuccess] = useState(false);
  const [webViewReady, setWebViewReady] = useState(false);
  const webViewRef = useRef<WebView>(null);
  const [loadingPayment, setLoadingPayment] = useState(false);
  const [holderCpf, setHolderCpf] = useState("");
  const [holderName, setHolderName] = useState("");
  const handleWebViewLoad = () => {
    console.log("WebView carregado!");
    setWebViewReady(true);
  };

  const webViewHtml = `
  <!DOCTYPE html>
    <html lang="en">
    <head>
      <meta charset="UTF-8" />
      <meta name="viewport" content="width=device-width, initial-scale=1.0" />
      <script src="https://cdn.jsdelivr.net/npm/payment-token-efi/dist/payment-token-efi-umd.min.js"></script>
    </head>
    <body>
      <script>
        // Override console.log to send messages to React Native
        const originalConsoleLog = console.log;
        console.log = function() {
          originalConsoleLog.apply(console, arguments);
          window.ReactNativeWebView.postMessage(JSON.stringify({
            type: 'console',
            message: Array.from(arguments).map(arg => 
              typeof arg === 'object' ? JSON.stringify(arg) : arg
            ).join(' ')
          }));
        };

        const originalConsoleError = console.error;
        console.error = function() {
          originalConsoleError.apply(console, arguments);
          window.ReactNativeWebView.postMessage(JSON.stringify({
            type: 'console',
            message: Array.from(arguments).map(arg => 
              typeof arg === 'object' ? JSON.stringify(arg) : arg
            ).join(' ')
          }));
        };

        
        
        async function generateToken(data) {
          try {
            console.log('Iniciando geração de token com dados:', JSON.stringify(data));
            const result = await EfiPay.CreditCard
              .setAccount("")
              .setEnvironment("production")
              .setCreditCardData(data)
              .getPaymentToken();

            console.log('Token gerado com sucesso:', result);
            window.ReactNativeWebView.postMessage(
              JSON.stringify({
                type: 'payment',
                payment_token: result.payment_token,
                card_mask: result.card_mask,
              })
            );
          } catch (error) {
            console.error('Erro detalhado:', error);
            window.ReactNativeWebView.postMessage(
              JSON.stringify({
                type: 'error',
                error: {
                  code: error.code || 'unknown',
                  name: error.error || 'Unknown Error',
                  message: error.error_description || error.message || 'Erro desconhecido',
                  details: error
                },
              })
            );
          }
        }
      </script>
    </body>
    </html>
`;

  const encodedHtml = `data:text/html;base64,${btoa(unescape(encodeURIComponent(webViewHtml)))}`;

  const { control, handleSubmit } = useForm<CreatePaymentWithCreditCardDto>({
    mode: "onChange",
    defaultValues: {
      holderName: "",
      cardNumber: "",
      expirationDate: "",
      ccv: "",
      cpfCnpj: "",
    },
  });

  const identifyBrand = async (cardNumber: string) => {
    try {
      const brand = await efiPay.CreditCard.setCardNumber(
        cardNumber
      ).verifyCardBrand();

      return brand;
    } catch (error) {
      console.log("Erro ao identificar a bandeira do cartão: ", error);
      return null;
    }
  };

  const sendPaymentDataToWebView = async (
    data: CreatePaymentWithCreditCardDto
  ) => {
    const brand = await identifyBrand(data.cardNumber);

    const cardData = {
      brand: brand,
      number: data.cardNumber.replace(/\D/g, ""),
      cvv: data.ccv.replace(/\D/g, ""),
      expirationMonth: data.expirationDate.split("/")[0],
      expirationYear: data.expirationDate.split("/")[1],
      holderName: data.holderName,
      holderDocument: data.cpfCnpj.replace(/\D/g, ""),
      reuse: false,
    };

    const script = `
      generateToken(${JSON.stringify(cardData)});
    `;

    if (!webViewRef.current) {
      console.log("WebView não está disponível ou não foi carregada");
      return;
    }

    webViewRef.current.injectJavaScript(script);
  };

  const handleWebViewMessage = async (event: any) => {
    try {
      const data = JSON.parse(event.nativeEvent.data);

      if (data.type === 'console') {
        console.log('WebView Log:', data.message);
        return;
      }

      if (data.type === 'payment' && data.payment_token && profile) {
        console.log("cpf: ", holderCpf);
        console.log("Name: ", holderName);
        const { data: paymentData } = await ticketCreditCard({
          eventId: ticket.eventId,
          ticketId: ticket.id,
          customerId: profile.id,
          selectedTicketDays: daysSelect.map((day) => day.id),
          creditCardToken: data.payment_token,
          isPackage: ticket.isPackage,
          isSuperPackage: ticket.isSuperPackage || false,
          holderName: holderName,
          holderCPF: holderCpf,
          holderEmail: profile.email,
          holderPhone: profile.mobilePhone,
          metadata: metaData,
        });

        const statusPayment = paymentData.data.status;
        console.log("statusPayment: ", statusPayment);

        if (statusPayment === "approved") {
          setSuccess(true);
        }
      } else if (data.type === 'error') {
        console.error("Erro detalhado:", data.error);
        Alert.alert(
          "Erro no Pagamento",
          `Código: ${data.error.code}\nMensagem: ${data.error.message}`
        );
      }
    } catch (error) {
      console.error("Erro ao processar mensagem:", error);
      Alert.alert("Erro", "Erro ao processar o pagamento");
    } finally {
      setLoadingPayment(false);
    }
  };

  const handleCloseSuccess = () => {
    router.replace("/(world-cup)/my-buys");
    setSuccess(false);
  };

  const handleCreatePayment = (data: CreatePaymentWithCreditCardDto) => {
    setHolderCpf(data.cpfCnpj.replace(/\D/g, ""));
    setHolderName(data.holderName);
    setLoadingPayment(true);
    sendPaymentDataToWebView(data);
  };

  useEffect(() => {
    handleGetProfile();
  }, []);

  return (
    <SafeAreaView
      style={{
        flex: 1,
        backgroundColor: theme.COLORS.WHITE,
        paddingTop: Platform.OS === "android" ? StatusBar.currentHeight : 0,
      }}
    >
      <AppBar title="Insira seus dados" onBack />
      {getLoading ? (
        <ActivityIndicator size="large" color={theme.COLORS.BLUE_800} />
      ) : (
        <KeyboardAwareScrollView
          style={{
            padding: 16,
            height: "100%",
          }}
          showsVerticalScrollIndicator={false}
        >
          <View
            style={{
              marginBottom: 16,
              gap: 16,
              paddingBottom: 8,
              borderBottomWidth: 1,
              borderBottomColor: theme.COLORS.GRAY_300,
            }}
          >
            <InputTextMaskController
              control={control}
              name="cardNumber"
              type="credit-card"
              label="Número do cartão*"
              placeholder="Ex: 1234 1234 1234 1234"
              keyboardType="numeric"
              rules={{
                required: "Campo obrigatório",
                validate: (value) => {
                  if (!value) {
                    return "Campo obrigatório";
                  }

                  if (String(value).length < 19) {
                    return "Número do cartão inválido";
                  }
                  return true;
                },
              }}
            />
            <InputTextController
              control={control}
              name="holderName"
              placeholder="Ex: João da Silva"
              label="Nome no cartão*"
              rules={{
                required: "Campo obrigatório",
              }}
            />

            <InputTextMaskController
              control={control}
              name="cpfCnpj"
              type="cpf"
              label="CPF do titular*"
              rules={{
                required: "O campo CPF é obrigatório",
                minLength: { message: "O campo esta incorreto!", value: 14 },
                validate: (value) => {
                  if (value && !validateCpf(String(value))) {
                    return "CPF inválido";
                  }

                  return true;
                },
              }}
              placeholder="Ex: 123.456.789-00"
              keyboardType="numeric"
            />

            <View style={{ flexDirection: "row", gap: 16 }}>
              <View style={{ flex: 2 }}>
                <InputTextMaskController
                  control={control}
                  name="expirationDate"
                  type="custom"
                  options={{
                    mask: "99/9999",
                  }}
                  label="Data de validade*"
                  placeholder="MM/AAAA"
                  keyboardType="numeric"
                  rules={{
                    required: "Campo obrigatório",
                    validate: (value) => {
                      if (!value) {
                        return "Campo obrigatório";
                      }

                      if (String(value).length < 5) {
                        return "Data de validade inválida";
                      }
                      return true;
                    },
                  }}
                />
              </View>
              <View style={{ flex: 1 }}>
                <InputTextController
                  control={control}
                  name="ccv"
                  placeholder="Ex: 123"
                  label="CCV*"
                  maxLength={3}
                  keyboardType="numeric"
                  rules={{
                    required: "Campo obrigatório",
                    validate: (value) => {
                      if (!value) {
                        return "Campo obrigatório";
                      }

                      if (String(value).length < 3) {
                        return "CCV inválido";
                      }
                      return true;
                    },
                  }}
                />
              </View>
            </View>
          </View>
          <View
            style={{
              marginBottom: 32,
            }}
          >
            <Button
              loading={loadingPayment}
              disabled={loadingPayment}
              title="Finalizar compra"
              onPress={handleSubmit(handleCreatePayment)}
            />
          </View>
        </KeyboardAwareScrollView>
      )}
      <BottomSheet
        onPress={handleCloseSuccess}
        isVisible={success}
        type="alert"
        size="60%"
      >
        <View
          style={{
            padding: 16,
            justifyContent: "center",
            alignItems: "center",
            gap: 16,
          }}
        >
          <Message
            title="Seu pagamento foi confirmado"
            type="success"
            onPress={handleCloseSuccess}
          />
          <Text
            text="Seu pagamento foi confirmado com sucesso, e o seu ticket foi gerado."
            type="SM"
            align="center"
          />
          <Button
            title="Ir para minhas compras"
            variant="primary"
            onPress={handleCloseSuccess}
          />
        </View>
      </BottomSheet>
      <WebView
        ref={webViewRef}
        source={{ uri: encodedHtml }}
        style={{ flex: 1, marginTop: 100 }}
        onMessage={handleWebViewMessage}
        onLoadEnd={handleWebViewLoad}
        originWhitelist={["*"]}
        javaScriptEnabled={true}
        domStorageEnabled={true}
        allowUniversalAccessFromFileURLs={true}
        allowFileAccess={true}
        allowFileAccessFromFileURLs={true}
        mixedContentMode="always"
        webviewDebuggingEnabled={true}
      />
    </SafeAreaView>
  );
}
